@@ -4,12 +4,15 @@ import hashlib
 import shutil
 import pickle
 import base64
-from multiprocessing import Process, JoinableQueue
+from multiprocessing import Process, JoinableQueue, Manager
 import subprocess
 from collections import Counter
 import configparser
 config = configparser.ConfigParser()
 import csv
+import random
+import json
+import glob
 
 def write2path(lines,path='',encoding='utf-8',method='lines',append='w',wrap=False):
     if not path: path=getTime()+'.txt'
@@ -71,11 +74,15 @@ def read_pkl(path):
         obj = pickle.load(fr)
         return obj
 
-def getTime(timestamp=''):
-    if timestamp:
-        return time.strftime("%y-%m-%d %H:%M:%S", time.localtime(timestamp))
+def getTime(timestamp='',format='y'):
+    if format=='y':
+        format_str="%y-%m-%d %H:%M:%S"
     else:
-        return time.strftime("%y-%m-%d %H:%M:%S", time.localtime())
+        format_str="%Y-%m-%d %H:%M:%S"
+    if timestamp:
+        return time.strftime(format_str, time.localtime(timestamp))
+    else:
+        return time.strftime(format_str, time.localtime())
 
 def getTimestamp():
     return time.time()
@@ -122,21 +129,37 @@ def getfilesize(filePath):
     fsize = fsize / (1000 * 1000)
     return round(fsize, 2)
 
-def listDir(rootDir,only_file=False,only_folder=False):
+def listDir(rootDir,only_file=False,only_folder=False,recursive=False):
     list_filepath = []
-    for filename in os.listdir(rootDir):
-        pathname = os.path.join(rootDir, filename)
-        if filename == '.DS_Store':
-            continue
-        if only_file:
-            if os.path.isfile(pathname):
-                list_filepath.append(pathname)
-            continue
-        if only_folder:
-            if os.path.isdir(pathname):
-                list_filepath.append(pathname)
-            continue
-        list_filepath.append(pathname)
+    if recursive:
+        all_file=glob.glob(os.path.join(rootDir, '**'), recursive=True)
+        for pathname in all_file:
+            filename = os.path.basename(pathname)
+            if filename == '.DS_Store':
+                continue
+            if only_file:
+                if os.path.isfile(pathname):
+                    list_filepath.append(pathname)
+                continue
+            if only_folder:
+                if os.path.isdir(pathname):
+                    list_filepath.append(pathname)
+                continue
+            list_filepath.append(pathname)
+    else:
+        for filename in os.listdir(rootDir):
+            pathname = os.path.join(rootDir, filename)
+            if filename == '.DS_Store':
+                continue
+            if only_file:
+                if os.path.isfile(pathname):
+                    list_filepath.append(pathname)
+                continue
+            if only_folder:
+                if os.path.isdir(pathname):
+                    list_filepath.append(pathname)
+                continue
+            list_filepath.append(pathname)
     return list_filepath
 
 def makedir(dir_path,delete_exists=False):
@@ -214,40 +237,61 @@ def func_time(func):
     end_time= time.time()
     print(f'{func.__name__} 耗时:{int(end_time-begin_time)}')
 
-def run_multi_task(all_items,user_func,done_path='',cpu_num=os.cpu_count(),print_log=False):
-    begin_time = time.time()
-    if done_path:
-        touchfile(done_path)
-        done_items = [f.strip() for f in read_path(done_path)]
-    else:
-        done_items = []
-    undone_items = list(set(all_items) - set(done_items))
+def run_multi_task(all_items, user_func, cpu_num=os.cpu_count()):
     print(f'检测到全部:{len(all_items)}个')
-    print(f'检测到已完成的:{len(done_items)}个')
-    print(f'重新计算需要处理的个数为:{len(undone_items)}个')
     print(f'启动{cpu_num}个进程,开始运行')
 
-    q = JoinableQueue()
-    for item in undone_items:
-        q.put(item)
+    if len(all_items)>32000:
+        print('数组个数大于32000，无法运行')
+        return []
+    else:
+        begin_time = time.time()
+        q = JoinableQueue()
+        for item in all_items:
+            q.put(item)
 
-    def func_task(q):
-        while True:
-            item = q.get()
-            user_func(item)
-            if done_path:
-                append2path(f'{item}',done_path)
+        def func_task(q, res_list):
+            while True:
+                item = q.get()
+                res_list.append(user_func(item))
+                q.task_done()
 
-            q.task_done()
-            if print_log:
-                print(f'当前:{item}')
+        res_list = Manager().list()
+        for i in range(cpu_num):
+            p = Process(target=func_task, args=(q, res_list))
+            p.daemon = True
+            p.start()
+        q.join()
+        print(f'全部已完成，用时:{float(time.time() - begin_time)}')
+        return res_list
 
-    for i in range(cpu_num):
-        p = Process(target=func_task, args=(q,))
-        p.daemon = True
-        p.start()
-    q.join()
-    print(f'全部已完成，用时:{float(time.time() - begin_time)}')
+def run_multi_task_in_order(all_items, user_func, cpu_num=os.cpu_count()):
+    print(f'检测到全部:{len(all_items)}个')
+    print(f'启动{cpu_num}个进程,开始运行')
+
+    if len(all_items)>32000:
+        print('数组个数大于32000，无法运行')
+        return {}
+    else:
+        begin_time = time.time()
+        q = JoinableQueue()
+        for index,data in enumerate(all_items):
+            q.put((index,data))
+
+        def func_task(q, res_dict):
+            while True:
+                item = q.get()
+                res_dict[item[0]] = user_func(item[1])
+                q.task_done()
+
+        res_dict = Manager().dict()
+        for i in range(cpu_num):
+            p = Process(target=func_task, args=(q, res_dict))
+            p.daemon = True
+            p.start()
+        q.join()
+        print(f'全部已完成，用时:{float(time.time() - begin_time)}')
+        return res_dict
 
 def read_config(config_path,config_name,section='default',type='str'):
     config = configparser.ConfigParser()
@@ -311,3 +355,37 @@ def write_csv(csv_path,header=[],rows=[],method="list",encoding='utf-8',append='
             writer = csv.DictWriter(f, header)
             writer.writeheader()
             writer.writerows(rows)
+
+def rand_int(min,max):
+    return random.randint(min, max)
+
+def rand_choice(lst):
+    result=random.choice(lst)
+    return result
+
+def rand_choices(lst,k=1):
+    result=random.choices(lst,k=k)
+    return result
+
+def rand_sample(lst, k=1):
+    result=random.sample(lst, k)
+    return result
+
+def rand_shuffle(lst):
+    random.shuffle(lst)
+    return lst
+
+def json_load(fp):
+    return json.load(fp)
+
+def json_loads(json_str):
+    return json.loads(json_str)
+
+def json_dump(json_data, fp):
+    return json.dump(json_data, fp)
+
+def json_dumps(json_data):
+    return json.dumps(json_data)
+
+def sleep(secs):
+    time.sleep(secs)
